/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_ROADEDGEINFO_H
#define DPC_IMPL_TYPE_ROADEDGEINFO_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint8.h"

namespace dpc {
struct RoadEdgeInfo {
    ::Float road_edge_c0{ 0.0F };
    ::Float road_edge_c1{ 0.0F };
    ::Float road_edge_c2{ 0.0F };
    ::Float road_edge_c3{ 0.0F };
    ::Float road_edge_viewrange{ 0.0F };
    ::UInt8 road_edge_typ{ 0U };
    ::Float road_edge_existnc_problty{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(road_edge_c0);
        fun(road_edge_c1);
        fun(road_edge_c2);
        fun(road_edge_c3);
        fun(road_edge_viewrange);
        fun(road_edge_typ);
        fun(road_edge_existnc_problty);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(road_edge_c0);
        fun(road_edge_c1);
        fun(road_edge_c2);
        fun(road_edge_c3);
        fun(road_edge_viewrange);
        fun(road_edge_typ);
        fun(road_edge_existnc_problty);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("road_edge_c0", road_edge_c0);
        fun("road_edge_c1", road_edge_c1);
        fun("road_edge_c2", road_edge_c2);
        fun("road_edge_c3", road_edge_c3);
        fun("road_edge_viewrange", road_edge_viewrange);
        fun("road_edge_typ", road_edge_typ);
        fun("road_edge_existnc_problty", road_edge_existnc_problty);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("road_edge_c0", road_edge_c0);
        fun("road_edge_c1", road_edge_c1);
        fun("road_edge_c2", road_edge_c2);
        fun("road_edge_c3", road_edge_c3);
        fun("road_edge_viewrange", road_edge_viewrange);
        fun("road_edge_typ", road_edge_typ);
        fun("road_edge_existnc_problty", road_edge_existnc_problty);
    }

    friend bool operator==(const ::dpc::RoadEdgeInfo& lhs, const ::dpc::RoadEdgeInfo& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.road_edge_c0 - rhs.road_edge_c0)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.road_edge_c1 - rhs.road_edge_c1)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.road_edge_c2 - rhs.road_edge_c2)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.road_edge_c3 - rhs.road_edge_c3)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.road_edge_viewrange - rhs.road_edge_viewrange)) < DBL_EPSILON) && (lhs.road_edge_typ == rhs.road_edge_typ) && (fabs(static_cast<double>(lhs.road_edge_existnc_problty - rhs.road_edge_existnc_problty)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_ROADEDGEINFO_H
